@@ -28,7 +28,7 @@ namespace xt
 
     namespace detail
     {
-        inline xbuffer_adaptor<int> r_shape_to_buffer_adaptor(SEXP exp)
+        inline xbuffer_adaptor<int*> r_shape_to_buffer_adaptor(SEXP exp)
         {
             if (Rf_isNull(Rf_getAttrib(exp, R_DimSymbol)))
             {
@@ -38,11 +38,11 @@ namespace xt
 
             SEXP shape_sexp = Rf_getAttrib(exp, R_DimSymbol);
             std::size_t n = (std::size_t)Rf_xlength(shape_sexp);
-            return xbuffer_adaptor<int>(
+            return xbuffer_adaptor<int*>(
                 Rcpp::internal::r_vector_start<INTSXP>(shape_sexp), n);
         }
 
-        inline xbuffer_adaptor<int> r_shape_to_buffer_adaptor(SEXP exp, std::size_t n)
+        inline xbuffer_adaptor<int*> r_shape_to_buffer_adaptor(SEXP exp, std::size_t n)
         {
             if (Rf_isNull(Rf_getAttrib(exp, R_DimSymbol)))
             {
@@ -55,7 +55,7 @@ namespace xt
             {
                 throw std::runtime_error("Could not convert shape. Dimensions don't match.");
             }
-            return xbuffer_adaptor<int>(
+            return xbuffer_adaptor<int*>(
                 Rcpp::internal::r_vector_start<INTSXP>(shape_sexp), n);
         }
     }
@@ -106,8 +106,8 @@ namespace xt
         static constexpr layout_type static_layout = layout_type::column_major;
         static constexpr bool contiguous_layout = true;
 
-        void reshape(const shape_type& shape);
-        void reshape(const shape_type& shape, const strides_type& strides);
+        template <class S>
+        void reshape(S&& shape);
 
         layout_type layout() const;
 
@@ -174,28 +174,16 @@ namespace xt
      * @param shape the new shape
      */
     template <class D>
-    inline void rcontainer<D>::reshape(const shape_type& shape)
+    template <class S>
+    inline void rcontainer<D>::reshape(S&& shape)
     {
-        if (shape.size() != this->dimension() || !std::equal(shape.begin(), shape.end(), this->shape().begin()))
+        if (shape.size() != this->dimension() || !std::equal(std::begin(shape), std::end(shape), this->shape().cbegin()))
         {
-            strides_type strides = xtl::make_sequence<strides_type>(shape.size(), size_type(1));
-            compute_strides(shape, layout_type::column_major, strides);
-            reshape(shape, strides);
+            derived_type tmp(std::forward<S>(shape));
+            *static_cast<derived_type*>(this) = std::move(tmp);
         }
     }
 
-    /**
-     * Reshapes the container.
-     * @param shape the new shape
-     * @param strides the new strides
-     */
-    template <class D>
-    inline void rcontainer<D>::reshape(const shape_type& shape, const strides_type& strides)
-    {
-        derived_type tmp(shape);
-        *static_cast<derived_type*>(this) = std::move(tmp);
-    }
-
     template <class D>
     inline layout_type rcontainer<D>::layout() const
     {
