@@ -6,8 +6,8 @@
 * The full license is in the file LICENSE, distributed with this software. *
 ****************************************************************************/
 
-#ifndef XINFO_HPP
-#define XINFO_HPP
+#ifndef XTENSOR_INFO_HPP
+#define XTENSOR_INFO_HPP
 
 #include <string>
 
@@ -17,7 +17,7 @@ namespace xt
     struct static_string
     {
         template <std::size_t N>
-        constexpr static_string(const char (&a)[N]) noexcept
+        explicit constexpr static_string(const char (&a)[N]) noexcept
             : data(a), size(N - 1)
         {
         }
@@ -62,14 +62,6 @@ namespace xt
     {
         std::string s;
         using shape_type = typename T::shape_type;
-        if (detail::is_array<shape_type>::value)
-        {
-            s += "Type: xtensor, fixed dimension " + std::to_string(t.dimension());
-        }
-        else
-        {
-            s += "Type: xarray, dimension " + std::to_string(t.dimension());
-        }
         s += "\nValue type: " + type_to_string<typename T::value_type>();
         s += "\nLayout: ";
         if (t.layout() == layout_type::row_major)
@@ -116,4 +108,3 @@ namespace xt
 }
 
 #endif
-
