@@ -6,8 +6,10 @@
 * The full license is in the file LICENSE, distributed with this software. *
 ****************************************************************************/
 
-#ifndef XOFFSETVIEW_HPP
-#define XOFFSETVIEW_HPP
+#ifndef XTENSOR_OFFSETVIEW_HPP
+#define XTENSOR_OFFSETVIEW_HPP
+
+#include "xtl/xcomplex.hpp"
 
 #include "xtensor/xfunctorview.hpp"
 
@@ -27,7 +29,7 @@ namespace xt
             template <class T>
             decltype(auto) operator()(T&& t) const
             {
-                return forward_offset<M, I>(t);
+                return xtl::forward_offset<M, I>(t);
             }
         };
     }
