@@ -12,6 +12,7 @@
 #include <memory>
 #include <vector>
 
+#include "xexpression.hpp"
 #include "xlayout.hpp"
 #include "xstorage.hpp"
 #include "xtensor_config.hpp"
@@ -24,10 +25,12 @@ namespace xt
     template <class C>
     struct xcontainer_inner_types;
 
-    template <class EC, layout_type L = DEFAULT_LAYOUT,
+    template <class EC,
+              layout_type L = DEFAULT_LAYOUT,
               class SC = DEFAULT_SHAPE_CONTAINER(typename EC::value_type,
                                                  typename EC::allocator_type,
-                                                 std::allocator<typename EC::size_type>)>
+                                                 std::allocator<typename EC::size_type>),
+              class Tag = xtensor_expression_tag>
     class xarray_container;
 
     /**
@@ -50,12 +53,36 @@ namespace xt
      * @tparam A The allocator of the container holding the elements.
      * @tparam SA The allocator of the containers holding the shape and the strides.
      */
-    template <class T, layout_type L = DEFAULT_LAYOUT,
+    template <class T,
+              layout_type L = DEFAULT_LAYOUT,
               class A = std::allocator<T>,
               class SA = std::allocator<typename std::vector<T, A>::size_type>>
     using xarray = xarray_container<DEFAULT_DATA_CONTAINER(T, A), L, DEFAULT_SHAPE_CONTAINER(T, A, SA)>;
 
-    template <class EC, std::size_t N, layout_type L = DEFAULT_LAYOUT>
+    template <class EC,
+              layout_type L = DEFAULT_LAYOUT,
+              class SC = std::vector<typename EC::size_type>,
+              class Tag = xtensor_expression_tag>
+    class xarray_adaptor;
+
+    /**
+     * @typedef xarray_optional
+     * Alias template on xarray_container for handling missing values
+     *
+     * @tparam T The value type of the elements.
+     * @tparam L The layout_type of the container (default: row_major).
+     * @tparam A The allocator of the container holding the elements.
+     * @tparam BA The allocator of the container holding the missing flags.
+     * @tparam SA The allocator of the containers holding the shape and the strides.
+     */
+    template <class T,
+              layout_type L = DEFAULT_LAYOUT,
+              class A = std::allocator<T>,
+              class BC = xtl::xdynamic_bitset<std::size_t>,
+              class SA = std::allocator<typename std::vector<T, A>::size_type>>
+    using xarray_optional = xarray_container<xtl::xoptional_vector<T, A, BC>, L, DEFAULT_SHAPE_CONTAINER(T, A, SA), xoptional_expression_tag>;
+
+    template <class EC, std::size_t N, layout_type L = DEFAULT_LAYOUT, class Tag = xtensor_expression_tag>
     class xtensor_container;
 
     /**
@@ -81,24 +108,8 @@ namespace xt
     template <class T, std::size_t N, layout_type L = DEFAULT_LAYOUT, class A = std::allocator<T>>
     using xtensor = xtensor_container<DEFAULT_DATA_CONTAINER(T, A), N, L>;
 
-    template <class CT, class... S>
-    class xview;
-
-    /**
-     * @typedef xarray_optional
-     * Alias template on xarray_container for handling missing values
-     *
-     * @tparam T The value type of the elements.
-     * @tparam L The layout_type of the container (default: row_major).
-     * @tparam A The allocator of the container holding the elements.
-     * @tparam BA The allocator of the container holding the missing flags.
-     * @tparam SA The allocator of the containers holding the shape and the strides.
-     */
-    template <class T, layout_type L = DEFAULT_LAYOUT,
-              class A = std::allocator<T>,
-              class BA = std::allocator<bool>,
-              class SA = std::allocator<typename std::vector<T, A>::size_type>>
-    using xarray_optional = xarray_container<xtl::xoptional_vector<T, A, BA>, L, DEFAULT_SHAPE_CONTAINER(T, A, SA)>;
+    template <class EC, std::size_t N, layout_type L = DEFAULT_LAYOUT, class Tag = xtensor_expression_tag>
+    class xtensor_adaptor;
 
     /**
      * @typedef xtensor_optional
@@ -110,8 +121,15 @@ namespace xt
      * @tparam A The allocator of the containers holding the elements.
      * @tparam BA The allocator of the container holding the missing flags.
      */
-    template <class T, std::size_t N, layout_type L = DEFAULT_LAYOUT, class A = std::allocator<T>, class BA = std::allocator<bool>>
-    using xtensor_optional = xtensor_container<xtl::xoptional_vector<T, A, BA>, N, L>;
+    template <class T,
+              std::size_t N,
+              layout_type L = DEFAULT_LAYOUT,
+              class A = std::allocator<T>,
+              class BC = xtl::xdynamic_bitset<std::size_t>>
+    using xtensor_optional = xtensor_container<xtl::xoptional_vector<T, A, BC>, N, L, xoptional_expression_tag>;
+
+    template <class CT, class... S>
+    class xview;
 
     namespace check_policy
     {
